@@ -1,14 +1,22 @@
 """Coders for individual Variable objects."""
+from __future__ import annotations
+
 import warnings
+from collections.abc import Hashable, MutableMapping
 from functools import partial
-from typing import Any, Hashable
+from typing import TYPE_CHECKING, Any, Callable, Union
 
 import numpy as np
 import pandas as pd
 
-from ..core import dtypes, duck_array_ops, indexing
-from ..core.pycompat import is_duck_dask_array
-from ..core.variable import Variable
+from xarray.core import dtypes, duck_array_ops, indexing
+from xarray.core.parallelcompat import get_chunked_array_type
+from xarray.core.pycompat import is_chunked_array
+from xarray.core.variable import Variable
+
+if TYPE_CHECKING:
+    T_VarTuple = tuple[tuple[Hashable, ...], Any, dict, dict]
+    T_Name = Union[Hashable, None]
 
 
 class SerializationWarning(RuntimeWarning):
@@ -32,15 +40,11 @@ class VariableCoder:
     variables in the underlying store.
     """
 
-    def encode(
-        self, variable: Variable, name: Hashable = None
-    ) -> Variable:  # pragma: no cover
+    def encode(self, variable: Variable, name: T_Name = None) -> Variable:
         """Convert an encoded variable to a decoded variable"""
         raise NotImplementedError()
 
-    def decode(
-        self, variable: Variable, name: Hashable = None
-    ) -> Variable:  # pragma: no cover
+    def decode(self, variable: Variable, name: T_Name = None) -> Variable:
         """Convert an decoded variable to a encoded variable"""
         raise NotImplementedError()
 
@@ -53,31 +57,95 @@ class _ElementwiseFunctionArray(indexing.ExplicitlyIndexedNDArrayMixin):
     Values are computed upon indexing or coercion to a NumPy array.
     """
 
-    def __init__(self, array, func, dtype):
-        assert not is_duck_dask_array(array)
+    def __init__(self, array, func: Callable, dtype: np.typing.DTypeLike):
+        assert not is_chunked_array(array)
         self.array = indexing.as_indexable(array)
         self.func = func
         self._dtype = dtype
 
     @property
-    def dtype(self):
+    def dtype(self) -> np.dtype:
         return np.dtype(self._dtype)
 
     def __getitem__(self, key):
         return type(self)(self.array[key], self.func, self.dtype)
 
-    def __array__(self, dtype=None):
-        return self.func(self.array)
+    def get_duck_array(self):
+        return self.func(self.array.get_duck_array())
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "{}({!r}, func={!r}, dtype={!r})".format(
             type(self).__name__, self.array, self.func, self.dtype
         )
 
 
-def lazy_elemwise_func(array, func, dtype):
-    """Lazily apply an element-wise function to an array.
+class NativeEndiannessArray(indexing.ExplicitlyIndexedNDArrayMixin):
+    """Decode arrays on the fly from non-native to native endianness
 
+    This is useful for decoding arrays from netCDF3 files (which are all
+    big endian) into native endianness, so they can be used with Cython
+    functions, such as those found in bottleneck and pandas.
+
+    >>> x = np.arange(5, dtype=">i2")
+
+    >>> x.dtype
+    dtype('>i2')
+
+    >>> NativeEndiannessArray(x).dtype
+    dtype('int16')
+
+    >>> indexer = indexing.BasicIndexer((slice(None),))
+    >>> NativeEndiannessArray(x)[indexer].dtype
+    dtype('int16')
+    """
+
+    __slots__ = ("array",)
+
+    def __init__(self, array) -> None:
+        self.array = indexing.as_indexable(array)
+
+    @property
+    def dtype(self) -> np.dtype:
+        return np.dtype(self.array.dtype.kind + str(self.array.dtype.itemsize))
+
+    def __getitem__(self, key) -> np.ndarray:
+        return np.asarray(self.array[key], dtype=self.dtype)
+
+
+class BoolTypeArray(indexing.ExplicitlyIndexedNDArrayMixin):
+    """Decode arrays on the fly from integer to boolean datatype
+
+    This is useful for decoding boolean arrays from integer typed netCDF
+    variables.
+
+    >>> x = np.array([1, 0, 1, 1, 0], dtype="i1")
+
+    >>> x.dtype
+    dtype('int8')
+
+    >>> BoolTypeArray(x).dtype
+    dtype('bool')
+
+    >>> indexer = indexing.BasicIndexer((slice(None),))
+    >>> BoolTypeArray(x)[indexer].dtype
+    dtype('bool')
+    """
+
+    __slots__ = ("array",)
+
+    def __init__(self, array) -> None:
+        self.array = indexing.as_indexable(array)
+
+    @property
+    def dtype(self) -> np.dtype:
+        return np.dtype("bool")
+
+    def __getitem__(self, key) -> np.ndarray:
+        return np.asarray(self.array[key], dtype=self.dtype)
+
+
+def lazy_elemwise_func(array, func: Callable, dtype: np.typing.DTypeLike):
+    """Lazily apply an element-wise function to an array.
     Parameters
     ----------
     array : any valid value of Variable._data
@@ -91,23 +159,23 @@ def lazy_elemwise_func(array, func, dtype):
     -------
     Either a dask.array.Array or _ElementwiseFunctionArray.
     """
-    if is_duck_dask_array(array):
-        import dask.array as da
+    if is_chunked_array(array):
+        chunkmanager = get_chunked_array_type(array)
 
-        return da.map_blocks(func, array, dtype=dtype)
+        return chunkmanager.map_blocks(func, array, dtype=dtype)
     else:
         return _ElementwiseFunctionArray(array, func, dtype)
 
 
-def unpack_for_encoding(var):
+def unpack_for_encoding(var: Variable) -> T_VarTuple:
     return var.dims, var.data, var.attrs.copy(), var.encoding.copy()
 
 
-def unpack_for_decoding(var):
+def unpack_for_decoding(var: Variable) -> T_VarTuple:
     return var.dims, var._data, var.attrs.copy(), var.encoding.copy()
 
 
-def safe_setitem(dest, key, value, name=None):
+def safe_setitem(dest, key: Hashable, value, name: T_Name = None):
     if key in dest:
         var_str = f" on variable {name!r}" if name else ""
         raise ValueError(
@@ -119,7 +187,9 @@ def safe_setitem(dest, key, value, name=None):
     dest[key] = value
 
 
-def pop_to(source, dest, key, name=None):
+def pop_to(
+    source: MutableMapping, dest: MutableMapping, key: Hashable, name: T_Name = None
+) -> Any:
     """
     A convenience function which pops a key k from source to dest.
     None values are not passed on.  If k already exists in dest an
@@ -132,7 +202,10 @@ def pop_to(source, dest, key, name=None):
 
 
 def _apply_mask(
-    data: np.ndarray, encoded_fill_values: list, decoded_fill_value: Any, dtype: Any
+    data: np.ndarray,
+    encoded_fill_values: list,
+    decoded_fill_value: Any,
+    dtype: np.typing.DTypeLike,
 ) -> np.ndarray:
     """Mask all matching values in a NumPy arrays."""
     data = np.asarray(data, dtype=dtype)
@@ -145,39 +218,41 @@ def _apply_mask(
 class CFMaskCoder(VariableCoder):
     """Mask or unmask fill values according to CF conventions."""
 
-    def encode(self, variable, name=None):
+    def encode(self, variable: Variable, name: T_Name = None):
         dims, data, attrs, encoding = unpack_for_encoding(variable)
 
         dtype = np.dtype(encoding.get("dtype", data.dtype))
         fv = encoding.get("_FillValue")
         mv = encoding.get("missing_value")
 
-        if (
-            fv is not None
-            and mv is not None
-            and not duck_array_ops.allclose_or_equiv(fv, mv)
-        ):
+        fv_exists = fv is not None
+        mv_exists = mv is not None
+
+        if not fv_exists and not mv_exists:
+            return variable
+
+        if fv_exists and mv_exists and not duck_array_ops.allclose_or_equiv(fv, mv):
             raise ValueError(
                 f"Variable {name!r} has conflicting _FillValue ({fv}) and missing_value ({mv}). Cannot encode data."
             )
 
-        if fv is not None:
+        if fv_exists:
             # Ensure _FillValue is cast to same dtype as data's
             encoding["_FillValue"] = dtype.type(fv)
             fill_value = pop_to(encoding, attrs, "_FillValue", name=name)
             if not pd.isnull(fill_value):
                 data = duck_array_ops.fillna(data, fill_value)
 
-        if mv is not None:
+        if mv_exists:
             # Ensure missing_value is cast to same dtype as data's
             encoding["missing_value"] = dtype.type(mv)
             fill_value = pop_to(encoding, attrs, "missing_value", name=name)
-            if not pd.isnull(fill_value) and fv is None:
+            if not pd.isnull(fill_value) and not fv_exists:
                 data = duck_array_ops.fillna(data, fill_value)
 
-        return Variable(dims, data, attrs, encoding)
+        return Variable(dims, data, attrs, encoding, fastpath=True)
 
-    def decode(self, variable, name=None):
+    def decode(self, variable: Variable, name: T_Name = None):
         dims, data, attrs, encoding = unpack_for_decoding(variable)
 
         raw_fill_values = [
@@ -211,11 +286,13 @@ class CFMaskCoder(VariableCoder):
                 )
                 data = lazy_elemwise_func(data, transform, dtype)
 
-        return Variable(dims, data, attrs, encoding)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
 
 
-def _scale_offset_decoding(data, scale_factor, add_offset, dtype):
-    data = np.array(data, dtype=dtype, copy=True)
+def _scale_offset_decoding(data, scale_factor, add_offset, dtype: np.typing.DTypeLike):
+    data = data.astype(dtype=dtype, copy=True)
     if scale_factor is not None:
         data *= scale_factor
     if add_offset is not None:
@@ -223,7 +300,7 @@ def _scale_offset_decoding(data, scale_factor, add_offset, dtype):
     return data
 
 
-def _choose_float_dtype(dtype, has_offset):
+def _choose_float_dtype(dtype: np.dtype, has_offset: bool) -> type[np.floating[Any]]:
     """Return a float dtype that can losslessly represent `dtype` values."""
     # Keep float32 as-is.  Upcast half-precision to single-precision,
     # because float16 is "intended for storage but not computation"
@@ -249,26 +326,27 @@ class CFScaleOffsetCoder(VariableCoder):
         decode_values = encoded_values * scale_factor + add_offset
     """
 
-    def encode(self, variable, name=None):
+    def encode(self, variable: Variable, name: T_Name = None) -> Variable:
         dims, data, attrs, encoding = unpack_for_encoding(variable)
 
         if "scale_factor" in encoding or "add_offset" in encoding:
             dtype = _choose_float_dtype(data.dtype, "add_offset" in encoding)
-            data = data.astype(dtype=dtype, copy=True)
-            if "add_offset" in encoding:
-                data -= pop_to(encoding, attrs, "add_offset", name=name)
-            if "scale_factor" in encoding:
-                data /= pop_to(encoding, attrs, "scale_factor", name=name)
+            data = duck_array_ops.astype(data, dtype=dtype, copy=True)
+        if "add_offset" in encoding:
+            data -= pop_to(encoding, attrs, "add_offset", name=name)
+        if "scale_factor" in encoding:
+            data /= pop_to(encoding, attrs, "scale_factor", name=name)
 
-        return Variable(dims, data, attrs, encoding)
+        return Variable(dims, data, attrs, encoding, fastpath=True)
 
-    def decode(self, variable, name=None):
-        dims, data, attrs, encoding = unpack_for_decoding(variable)
+    def decode(self, variable: Variable, name: T_Name = None) -> Variable:
+        _attrs = variable.attrs
+        if "scale_factor" in _attrs or "add_offset" in _attrs:
+            dims, data, attrs, encoding = unpack_for_decoding(variable)
 
-        if "scale_factor" in attrs or "add_offset" in attrs:
             scale_factor = pop_to(attrs, encoding, "scale_factor", name=name)
             add_offset = pop_to(attrs, encoding, "add_offset", name=name)
-            dtype = _choose_float_dtype(data.dtype, "add_offset" in attrs)
+            dtype = _choose_float_dtype(data.dtype, "add_offset" in encoding)
             if np.ndim(scale_factor) > 0:
                 scale_factor = np.asarray(scale_factor).item()
             if np.ndim(add_offset) > 0:
@@ -281,36 +359,40 @@ class CFScaleOffsetCoder(VariableCoder):
             )
             data = lazy_elemwise_func(data, transform, dtype)
 
-        return Variable(dims, data, attrs, encoding)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
 
 
 class UnsignedIntegerCoder(VariableCoder):
-    def encode(self, variable, name=None):
-        dims, data, attrs, encoding = unpack_for_encoding(variable)
-
+    def encode(self, variable: Variable, name: T_Name = None) -> Variable:
         # from netCDF best practices
         # https://www.unidata.ucar.edu/software/netcdf/docs/BestPractices.html
         #     "_Unsigned = "true" to indicate that
         #      integer data should be treated as unsigned"
-        if encoding.get("_Unsigned", "false") == "true":
+        if variable.encoding.get("_Unsigned", "false") == "true":
+            dims, data, attrs, encoding = unpack_for_encoding(variable)
+
             pop_to(encoding, attrs, "_Unsigned")
-            signed_dtype = np.dtype("i%s" % data.dtype.itemsize)
+            signed_dtype = np.dtype(f"i{data.dtype.itemsize}")
             if "_FillValue" in attrs:
                 new_fill = signed_dtype.type(attrs["_FillValue"])
                 attrs["_FillValue"] = new_fill
-            data = duck_array_ops.around(data).astype(signed_dtype)
+            data = duck_array_ops.astype(duck_array_ops.around(data), signed_dtype)
 
-        return Variable(dims, data, attrs, encoding)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
 
-    def decode(self, variable, name=None):
-        dims, data, attrs, encoding = unpack_for_decoding(variable)
+    def decode(self, variable: Variable, name: T_Name = None) -> Variable:
+        if "_Unsigned" in variable.attrs:
+            dims, data, attrs, encoding = unpack_for_decoding(variable)
 
-        if "_Unsigned" in attrs:
             unsigned = pop_to(attrs, encoding, "_Unsigned")
 
             if data.dtype.kind == "i":
                 if unsigned == "true":
-                    unsigned_dtype = np.dtype("u%s" % data.dtype.itemsize)
+                    unsigned_dtype = np.dtype(f"u{data.dtype.itemsize}")
                     transform = partial(np.asarray, dtype=unsigned_dtype)
                     data = lazy_elemwise_func(data, transform, unsigned_dtype)
                     if "_FillValue" in attrs:
@@ -318,7 +400,7 @@ class UnsignedIntegerCoder(VariableCoder):
                         attrs["_FillValue"] = new_fill
             elif data.dtype.kind == "u":
                 if unsigned == "false":
-                    signed_dtype = np.dtype("i%s" % data.dtype.itemsize)
+                    signed_dtype = np.dtype(f"i{data.dtype.itemsize}")
                     transform = partial(np.asarray, dtype=signed_dtype)
                     data = lazy_elemwise_func(data, transform, signed_dtype)
                     if "_FillValue" in attrs:
@@ -326,10 +408,110 @@ class UnsignedIntegerCoder(VariableCoder):
                         attrs["_FillValue"] = new_fill
             else:
                 warnings.warn(
-                    "variable %r has _Unsigned attribute but is not "
-                    "of integer type. Ignoring attribute." % name,
+                    f"variable {name!r} has _Unsigned attribute but is not "
+                    "of integer type. Ignoring attribute.",
                     SerializationWarning,
                     stacklevel=3,
                 )
 
-        return Variable(dims, data, attrs, encoding)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
+
+
+class DefaultFillvalueCoder(VariableCoder):
+    """Encode default _FillValue if needed."""
+
+    def encode(self, variable: Variable, name: T_Name = None) -> Variable:
+        dims, data, attrs, encoding = unpack_for_encoding(variable)
+        # make NaN the fill value for float types
+        if (
+            "_FillValue" not in attrs
+            and "_FillValue" not in encoding
+            and np.issubdtype(variable.dtype, np.floating)
+        ):
+            attrs["_FillValue"] = variable.dtype.type(np.nan)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
+
+    def decode(self, variable: Variable, name: T_Name = None) -> Variable:
+        raise NotImplementedError()
+
+
+class BooleanCoder(VariableCoder):
+    """Code boolean values."""
+
+    def encode(self, variable: Variable, name: T_Name = None) -> Variable:
+        if (
+            (variable.dtype == bool)
+            and ("dtype" not in variable.encoding)
+            and ("dtype" not in variable.attrs)
+        ):
+            dims, data, attrs, encoding = unpack_for_encoding(variable)
+            attrs["dtype"] = "bool"
+            data = duck_array_ops.astype(data, dtype="i1", copy=True)
+
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
+
+    def decode(self, variable: Variable, name: T_Name = None) -> Variable:
+        if variable.attrs.get("dtype", False) == "bool":
+            dims, data, attrs, encoding = unpack_for_decoding(variable)
+            # overwrite (!) dtype in encoding, and remove from attrs
+            # needed for correct subsequent encoding
+            encoding["dtype"] = attrs.pop("dtype")
+            data = BoolTypeArray(data)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
+
+
+class EndianCoder(VariableCoder):
+    """Decode Endianness to native."""
+
+    def encode(self):
+        raise NotImplementedError()
+
+    def decode(self, variable: Variable, name: T_Name = None) -> Variable:
+        dims, data, attrs, encoding = unpack_for_decoding(variable)
+        if not data.dtype.isnative:
+            data = NativeEndiannessArray(data)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
+
+
+class NonStringCoder(VariableCoder):
+    """Encode NonString variables if dtypes differ."""
+
+    def encode(self, variable: Variable, name: T_Name = None) -> Variable:
+        if "dtype" in variable.encoding and variable.encoding["dtype"] not in (
+            "S1",
+            str,
+        ):
+            dims, data, attrs, encoding = unpack_for_encoding(variable)
+            dtype = np.dtype(encoding.pop("dtype"))
+            if dtype != variable.dtype:
+                if np.issubdtype(dtype, np.integer):
+                    if (
+                        np.issubdtype(variable.dtype, np.floating)
+                        and "_FillValue" not in variable.attrs
+                        and "missing_value" not in variable.attrs
+                    ):
+                        warnings.warn(
+                            f"saving variable {name} with floating "
+                            "point data as an integer dtype without "
+                            "any _FillValue to use for NaNs",
+                            SerializationWarning,
+                            stacklevel=10,
+                        )
+                    data = np.around(data)
+                data = data.astype(dtype=dtype)
+            return Variable(dims, data, attrs, encoding, fastpath=True)
+        else:
+            return variable
+
+    def decode(self):
+        raise NotImplementedError()
