@@ -1,15 +1,22 @@
+from __future__ import annotations
+
 import warnings
 from collections import defaultdict
+from collections.abc import Hashable, Iterable, Mapping, MutableMapping
+from typing import TYPE_CHECKING, Any, Union
 
 import numpy as np
 import pandas as pd
 
-from .coding import strings, times, variables
-from .coding.variables import SerializationWarning, pop_to
-from .core import duck_array_ops, indexing
-from .core.common import contains_cftime_datetimes
-from .core.pycompat import is_duck_dask_array
-from .core.variable import IndexVariable, Variable, as_variable
+from xarray.coding import strings, times, variables
+from xarray.coding.variables import SerializationWarning, pop_to
+from xarray.core import indexing
+from xarray.core.common import (
+    _contains_datetime_like_objects,
+    contains_cftime_datetimes,
+)
+from xarray.core.pycompat import is_duck_dask_array
+from xarray.core.variable import IndexVariable, Variable
 
 CF_RELATED_DATA = (
     "bounds",
@@ -29,124 +36,23 @@ CF_RELATED_DATA_NEEDS_PARSING = (
 )
 
 
-class NativeEndiannessArray(indexing.ExplicitlyIndexedNDArrayMixin):
-    """Decode arrays on the fly from non-native to native endianness
+if TYPE_CHECKING:
+    from xarray.backends.common import AbstractDataStore
+    from xarray.core.dataset import Dataset
 
-    This is useful for decoding arrays from netCDF3 files (which are all
-    big endian) into native endianness, so they can be used with Cython
-    functions, such as those found in bottleneck and pandas.
-
-    >>> x = np.arange(5, dtype=">i2")
-
-    >>> x.dtype
-    dtype('>i2')
-
-    >>> NativeEndiannessArray(x).dtype
-    dtype('int16')
-
-    >>> indexer = indexing.BasicIndexer((slice(None),))
-    >>> NativeEndiannessArray(x)[indexer].dtype
-    dtype('int16')
-    """
-
-    __slots__ = ("array",)
-
-    def __init__(self, array):
-        self.array = indexing.as_indexable(array)
-
-    @property
-    def dtype(self):
-        return np.dtype(self.array.dtype.kind + str(self.array.dtype.itemsize))
-
-    def __getitem__(self, key):
-        return np.asarray(self.array[key], dtype=self.dtype)
+    T_VarTuple = tuple[tuple[Hashable, ...], Any, dict, dict]
+    T_Name = Union[Hashable, None]
+    T_Variables = Mapping[Any, Variable]
+    T_Attrs = MutableMapping[Any, Any]
+    T_DropVariables = Union[str, Iterable[Hashable], None]
+    T_DatasetOrAbstractstore = Union[Dataset, AbstractDataStore]
 
 
-class BoolTypeArray(indexing.ExplicitlyIndexedNDArrayMixin):
-    """Decode arrays on the fly from integer to boolean datatype
-
-    This is useful for decoding boolean arrays from integer typed netCDF
-    variables.
-
-    >>> x = np.array([1, 0, 1, 1, 0], dtype="i1")
-
-    >>> x.dtype
-    dtype('int8')
-
-    >>> BoolTypeArray(x).dtype
-    dtype('bool')
-
-    >>> indexer = indexing.BasicIndexer((slice(None),))
-    >>> BoolTypeArray(x)[indexer].dtype
-    dtype('bool')
-    """
-
-    __slots__ = ("array",)
-
-    def __init__(self, array):
-        self.array = indexing.as_indexable(array)
-
-    @property
-    def dtype(self):
-        return np.dtype("bool")
-
-    def __getitem__(self, key):
-        return np.asarray(self.array[key], dtype=self.dtype)
-
-
-def _var_as_tuple(var):
+def _var_as_tuple(var: Variable) -> T_VarTuple:
     return var.dims, var.data, var.attrs.copy(), var.encoding.copy()
 
 
-def maybe_encode_nonstring_dtype(var, name=None):
-    if "dtype" in var.encoding and var.encoding["dtype"] not in ("S1", str):
-        dims, data, attrs, encoding = _var_as_tuple(var)
-        dtype = np.dtype(encoding.pop("dtype"))
-        if dtype != var.dtype:
-            if np.issubdtype(dtype, np.integer):
-                if (
-                    np.issubdtype(var.dtype, np.floating)
-                    and "_FillValue" not in var.attrs
-                    and "missing_value" not in var.attrs
-                ):
-                    warnings.warn(
-                        "saving variable %s with floating "
-                        "point data as an integer dtype without "
-                        "any _FillValue to use for NaNs" % name,
-                        SerializationWarning,
-                        stacklevel=10,
-                    )
-                data = duck_array_ops.around(data)[...]
-            data = data.astype(dtype=dtype)
-        var = Variable(dims, data, attrs, encoding)
-    return var
-
-
-def maybe_default_fill_value(var):
-    # make NaN the fill value for float types:
-    if (
-        "_FillValue" not in var.attrs
-        and "_FillValue" not in var.encoding
-        and np.issubdtype(var.dtype, np.floating)
-    ):
-        var.attrs["_FillValue"] = var.dtype.type(np.nan)
-    return var
-
-
-def maybe_encode_bools(var):
-    if (
-        (var.dtype == bool)
-        and ("dtype" not in var.encoding)
-        and ("dtype" not in var.attrs)
-    ):
-        dims, data, attrs, encoding = _var_as_tuple(var)
-        attrs["dtype"] = "bool"
-        data = data.astype(dtype="i1", copy=True)
-        var = Variable(dims, data, attrs, encoding)
-    return var
-
-
-def _infer_dtype(array, name=None):
+def _infer_dtype(array, name: T_Name = None) -> np.dtype:
     """Given an object array with no missing values, infer its dtype from its
     first element
     """
@@ -157,8 +63,12 @@ def _infer_dtype(array, name=None):
         return np.dtype(float)
 
     element = array[(0,) * array.ndim]
-    if isinstance(element, (bytes, str)):
-        return strings.create_vlen_dtype(type(element))
+    # We use the base types to avoid subclasses of bytes and str (which might
+    # not play nice with e.g. hdf5 datatypes), such as those from numpy
+    if isinstance(element, bytes):
+        return strings.create_vlen_dtype(bytes)
+    elif isinstance(element, str):
+        return strings.create_vlen_dtype(str)
 
     dtype = np.array(element).dtype
     if dtype.kind != "O":
@@ -170,18 +80,19 @@ def _infer_dtype(array, name=None):
     )
 
 
-def ensure_not_multiindex(var, name=None):
+def ensure_not_multiindex(var: Variable, name: T_Name = None) -> None:
     if isinstance(var, IndexVariable) and isinstance(var.to_index(), pd.MultiIndex):
         raise NotImplementedError(
             "variable {!r} is a MultiIndex, which cannot yet be "
-            "serialized to netCDF files "
-            "(https://github.com/pydata/xarray/issues/1077). Use "
-            "reset_index() to convert MultiIndex levels into coordinate "
-            "variables instead.".format(name)
+            "serialized to netCDF files. Instead, either use reset_index() "
+            "to convert MultiIndex levels into coordinate variables instead "
+            "or use https://cf-xarray.readthedocs.io/en/latest/coding.html.".format(
+                name
+            )
         )
 
 
-def _copy_with_dtype(data, dtype):
+def _copy_with_dtype(data, dtype: np.typing.DTypeLike):
     """Create a copy of an array with the given dtype.
 
     We use this instead of np.array() to ensure that custom object dtypes end
@@ -192,11 +103,15 @@ def _copy_with_dtype(data, dtype):
     return result
 
 
-def ensure_dtype_not_object(var, name=None):
+def ensure_dtype_not_object(var: Variable, name: T_Name = None) -> Variable:
     # TODO: move this from conventions to backends? (it's not CF related)
     if var.dtype.kind == "O":
         dims, data, attrs, encoding = _var_as_tuple(var)
 
+        # leave vlen dtypes unchanged
+        if strings.check_vlen_dtype(data.dtype) is not None:
+            return var
+
         if is_duck_dask_array(data):
             warnings.warn(
                 "variable {} has data in the form of a dask array with "
@@ -217,6 +132,7 @@ def ensure_dtype_not_object(var, name=None):
             # There is no safe bit-pattern for NA in typical binary string
             # formats, we so can't set a fill_value. Unfortunately, this means
             # we can't distinguish between missing values and empty strings.
+            fill_value: bytes | str
             if strings.is_bytes_dtype(inferred_dtype):
                 fill_value = b""
             elif strings.is_unicode_dtype(inferred_dtype):
@@ -233,11 +149,13 @@ def ensure_dtype_not_object(var, name=None):
             data = _copy_with_dtype(data, dtype=_infer_dtype(data, name))
 
         assert data.dtype.kind != "O" or data.dtype.metadata
-        var = Variable(dims, data, attrs, encoding)
+        var = Variable(dims, data, attrs, encoding, fastpath=True)
     return var
 
 
-def encode_cf_variable(var, needs_copy=True, name=None):
+def encode_cf_variable(
+    var: Variable, needs_copy: bool = True, name: T_Name = None
+) -> Variable:
     """
     Converts an Variable into an Variable which follows some
     of the CF conventions:
@@ -265,13 +183,13 @@ def encode_cf_variable(var, needs_copy=True, name=None):
         variables.CFScaleOffsetCoder(),
         variables.CFMaskCoder(),
         variables.UnsignedIntegerCoder(),
+        variables.NonStringCoder(),
+        variables.DefaultFillvalueCoder(),
+        variables.BooleanCoder(),
     ]:
         var = coder.encode(var, name=name)
 
-    # TODO(shoyer): convert all of these to use coders, too:
-    var = maybe_encode_nonstring_dtype(var, name=name)
-    var = maybe_default_fill_value(var)
-    var = maybe_encode_bools(var)
+    # TODO(kmuehlbauer): check if ensure_dtype_not_object can be moved to backends:
     var = ensure_dtype_not_object(var, name=name)
 
     for attr_name in CF_RELATED_DATA:
@@ -280,16 +198,16 @@ def encode_cf_variable(var, needs_copy=True, name=None):
 
 
 def decode_cf_variable(
-    name,
-    var,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-    decode_endianness=True,
-    stack_char_dim=True,
-    use_cftime=None,
-    decode_timedelta=None,
-):
+    name: Hashable,
+    var: Variable,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+    decode_endianness: bool = True,
+    stack_char_dim: bool = True,
+    use_cftime: bool | None = None,
+    decode_timedelta: bool | None = None,
+) -> Variable:
     """
     Decodes a variable which may hold CF encoded information.
 
@@ -335,7 +253,10 @@ def decode_cf_variable(
     out : Variable
         A variable holding the decoded equivalent of var.
     """
-    var = as_variable(var)
+    # Ensure datetime-like Variables are passed through unmodified (GH 6453)
+    if _contains_datetime_like_objects(var):
+        return var
+
     original_dtype = var.dtype
 
     if decode_timedelta is None:
@@ -359,27 +280,23 @@ def decode_cf_variable(
     if decode_times:
         var = times.CFDatetimeCoder(use_cftime=use_cftime).decode(var, name=name)
 
-    dimensions, data, attributes, encoding = variables.unpack_for_decoding(var)
-    # TODO(shoyer): convert everything below to use coders
+    if decode_endianness and not var.dtype.isnative:
+        var = variables.EndianCoder().decode(var)
+        original_dtype = var.dtype
 
-    if decode_endianness and not data.dtype.isnative:
-        # do this last, so it's only done if we didn't already unmask/scale
-        data = NativeEndiannessArray(data)
-        original_dtype = data.dtype
+    var = variables.BooleanCoder().decode(var)
+
+    dimensions, data, attributes, encoding = variables.unpack_for_decoding(var)
 
     encoding.setdefault("dtype", original_dtype)
 
-    if "dtype" in attributes and attributes["dtype"] == "bool":
-        del attributes["dtype"]
-        data = BoolTypeArray(data)
-
     if not is_duck_dask_array(data):
         data = indexing.LazilyIndexedArray(data)
 
-    return Variable(dimensions, data, attributes, encoding=encoding)
+    return Variable(dimensions, data, attributes, encoding=encoding, fastpath=True)
 
 
-def _update_bounds_attributes(variables):
+def _update_bounds_attributes(variables: T_Variables) -> None:
     """Adds time attributes to time bounds variables.
 
     Variables handling time bounds ("Cell boundaries" in the CF
@@ -398,7 +315,8 @@ def _update_bounds_attributes(variables):
     # For all time variables with bounds
     for v in variables.values():
         attrs = v.attrs
-        has_date_units = "units" in attrs and "since" in attrs["units"]
+        units = attrs.get("units")
+        has_date_units = isinstance(units, str) and "since" in units
         if has_date_units and "bounds" in attrs:
             if attrs["bounds"] in variables:
                 bounds_attrs = variables[attrs["bounds"]].attrs
@@ -407,7 +325,7 @@ def _update_bounds_attributes(variables):
                     bounds_attrs.setdefault("calendar", attrs["calendar"])
 
 
-def _update_bounds_encoding(variables):
+def _update_bounds_encoding(variables: T_Variables) -> None:
     """Adds time encoding to time bounds variables.
 
     Variables handling time bounds ("Cell boundaries" in the CF
@@ -424,7 +342,7 @@ def _update_bounds_encoding(variables):
     """
 
     # For all time variables with bounds
-    for v in variables.values():
+    for name, v in variables.items():
         attrs = v.attrs
         encoding = v.encoding
         has_date_units = "units" in encoding and "since" in encoding["units"]
@@ -445,7 +363,7 @@ def _update_bounds_encoding(variables):
                 "and '{1}' will be determined independently "
                 "and may not be equal, counter to CF-conventions. "
                 "If this is a concern, specify a units encoding for "
-                "'{0}' before writing to a file.".format(v.name, attrs["bounds"]),
+                "'{0}' before writing to a file.".format(name, attrs["bounds"]),
                 UserWarning,
             )
 
@@ -458,16 +376,16 @@ def _update_bounds_encoding(variables):
 
 
 def decode_cf_variables(
-    variables,
-    attributes,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-    decode_coords=True,
-    drop_variables=None,
-    use_cftime=None,
-    decode_timedelta=None,
-):
+    variables: T_Variables,
+    attributes: T_Attrs,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+    decode_coords: bool = True,
+    drop_variables: T_DropVariables = None,
+    use_cftime: bool | None = None,
+    decode_timedelta: bool | None = None,
+) -> tuple[T_Variables, T_Attrs, set[Hashable]]:
     """
     Decode several CF encoded variables.
 
@@ -478,7 +396,7 @@ def decode_cf_variables(
         for d in v.dims:
             dimensions_used_by[d].append(v)
 
-    def stackable(dim):
+    def stackable(dim: Hashable) -> bool:
         # figure out if a dimension can be concatenated over
         if dim in variables:
             return False
@@ -509,16 +427,19 @@ def decode_cf_variables(
             and v.ndim > 0
             and stackable(v.dims[-1])
         )
-        new_vars[k] = decode_cf_variable(
-            k,
-            v,
-            concat_characters=concat_characters,
-            mask_and_scale=mask_and_scale,
-            decode_times=decode_times,
-            stack_char_dim=stack_char_dim,
-            use_cftime=use_cftime,
-            decode_timedelta=decode_timedelta,
-        )
+        try:
+            new_vars[k] = decode_cf_variable(
+                k,
+                v,
+                concat_characters=concat_characters,
+                mask_and_scale=mask_and_scale,
+                decode_times=decode_times,
+                stack_char_dim=stack_char_dim,
+                use_cftime=use_cftime,
+                decode_timedelta=decode_timedelta,
+            )
+        except Exception as e:
+            raise type(e)(f"Failed to decode variable {k!r}: {e}")
         if decode_coords in [True, "coordinates", "all"]:
             var_attrs = new_vars[k].attrs
             if "coordinates" in var_attrs:
@@ -561,23 +482,24 @@ def decode_cf_variables(
                         )
                     del var_attrs[attr_name]
 
-    if decode_coords and "coordinates" in attributes:
+    if decode_coords and isinstance(attributes.get("coordinates", None), str):
         attributes = dict(attributes)
-        coord_names.update(attributes.pop("coordinates").split())
+        crds = attributes.pop("coordinates")
+        coord_names.update(crds.split())
 
     return new_vars, attributes, coord_names
 
 
 def decode_cf(
-    obj,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-    decode_coords=True,
-    drop_variables=None,
-    use_cftime=None,
-    decode_timedelta=None,
-):
+    obj: T_DatasetOrAbstractstore,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+    decode_coords: bool = True,
+    drop_variables: T_DropVariables = None,
+    use_cftime: bool | None = None,
+    decode_timedelta: bool | None = None,
+) -> Dataset:
     """Decode the given Dataset or Datastore according to CF conventions into
     a new Dataset.
 
@@ -626,9 +548,11 @@ def decode_cf(
     -------
     decoded : Dataset
     """
-    from .backends.common import AbstractDataStore
-    from .core.dataset import Dataset
+    from xarray.backends.common import AbstractDataStore
+    from xarray.core.dataset import Dataset
 
+    vars: T_Variables
+    attrs: T_Attrs
     if isinstance(obj, Dataset):
         vars = obj._variables
         attrs = obj.attrs
@@ -663,12 +587,12 @@ def decode_cf(
 
 
 def cf_decoder(
-    variables,
-    attributes,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-):
+    variables: T_Variables,
+    attributes: T_Attrs,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+) -> tuple[T_Variables, T_Attrs]:
     """
     Decode a set of CF encoded variables and attributes.
 
@@ -751,10 +675,22 @@ def _encode_coordinates(variables, attributes, non_dim_coord_names):
                 f"'coordinates' found in both attrs and encoding for variable {name!r}."
             )
 
+        # if coordinates set to None, don't write coordinates attribute
+        if (
+            "coordinates" in attrs
+            and attrs.get("coordinates") is None
+            or "coordinates" in encoding
+            and encoding.get("coordinates") is None
+        ):
+            # make sure "coordinates" is removed from attrs/encoding
+            attrs.pop("coordinates", None)
+            encoding.pop("coordinates", None)
+            continue
+
         # this will copy coordinates from encoding to attrs if "coordinates" in attrs
         # after the next line, "coordinates" is never in encoding
         # we get support for attrs["coordinates"] for free.
-        coords_str = pop_to(encoding, attrs, "coordinates")
+        coords_str = pop_to(encoding, attrs, "coordinates") or attrs.get("coordinates")
         if not coords_str and variable_coordinates[name]:
             coordinates_text = " ".join(
                 str(coord_name)
