@@ -3,19 +3,20 @@ from __future__ import annotations
 import itertools
 import warnings
 from collections import Counter
-from typing import TYPE_CHECKING, Iterable, Literal, Sequence, Union
+from collections.abc import Iterable, Sequence
+from typing import TYPE_CHECKING, Literal, Union
 
 import pandas as pd
 
-from . import dtypes
-from .concat import concat
-from .dataarray import DataArray
-from .dataset import Dataset
-from .merge import merge
-from .utils import iterate_nested
+from xarray.core import dtypes
+from xarray.core.concat import concat
+from xarray.core.dataarray import DataArray
+from xarray.core.dataset import Dataset
+from xarray.core.merge import merge
+from xarray.core.utils import iterate_nested
 
 if TYPE_CHECKING:
-    from .types import CombineAttrsOptions, CompatOptions, JoinOptions
+    from xarray.core.types import CombineAttrsOptions, CompatOptions, JoinOptions
 
 
 def _infer_concat_order_from_positions(datasets):
@@ -52,7 +53,6 @@ def _infer_tile_ids_from_nested_list(entry, current_pos):
 
 
 def _ensure_same_types(series, dim):
-
     if series.dtype == object:
         types = set(series.map(type))
         if len(types) > 1:
@@ -79,17 +79,14 @@ def _ensure_same_types(series, dim):
 
 
 def _infer_concat_order_from_coords(datasets):
-
     concat_dims = []
     tile_ids = [() for ds in datasets]
 
     # All datasets have same variables because they've been grouped as such
     ds0 = datasets[0]
     for dim in ds0.dims:
-
         # Check if dim is a coordinate dimension
         if dim in ds0:
-
             # Need to read coordinate values to do ordering
             indexes = [ds._indexes.get(dim) for ds in datasets]
             if any(index is None for index in indexes):
@@ -104,7 +101,6 @@ def _infer_concat_order_from_coords(datasets):
             # If dimension coordinate values are same on every dataset then
             # should be leaving this dimension alone (it's just a "bystander")
             if not all(index.equals(indexes[0]) for index in indexes[1:]):
-
                 # Infer order datasets should be arranged in along this dim
                 concat_dims.append(dim)
 
@@ -260,7 +256,6 @@ def _combine_all_along_first_dim(
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "drop",
 ):
-
     # Group into lines of datasets which must be combined along dim
     # need to sort by _new_tile_id first for groupby to work
     # TODO: is the sorted need?
@@ -344,7 +339,6 @@ def _nested_combine(
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "drop",
 ):
-
     if len(datasets) == 0:
         return Dataset()
 
@@ -375,9 +369,8 @@ def _nested_combine(
     return combined
 
 
-# Define type for arbitrarily-nested list of lists recursively
-# Currently mypy cannot handle this but other linters can (https://stackoverflow.com/a/53845083/3154101)
-DATASET_HYPERCUBE = Union[Dataset, Iterable["DATASET_HYPERCUBE"]]  # type: ignore
+# Define type for arbitrarily-nested list of lists recursively:
+DATASET_HYPERCUBE = Union[Dataset, Iterable["DATASET_HYPERCUBE"]]
 
 
 def combine_nested(
@@ -669,7 +662,7 @@ def combine_by_coords(
     fill_value: object = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "no_conflicts",
-    datasets: Iterable[Dataset] = None,
+    datasets: Iterable[Dataset] | None = None,
 ) -> Dataset | DataArray:
     """
 
@@ -977,10 +970,9 @@ def combine_by_coords(
 
         # Perform the multidimensional combine on each group of data variables
         # before merging back together
-        concatenated_grouped_by_data_vars = []
-        for vars, datasets_with_same_vars in grouped_by_vars:
-            concatenated = _combine_single_variable_hypercube(
-                list(datasets_with_same_vars),
+        concatenated_grouped_by_data_vars = tuple(
+            _combine_single_variable_hypercube(
+                tuple(datasets_with_same_vars),
                 fill_value=fill_value,
                 data_vars=data_vars,
                 coords=coords,
@@ -988,7 +980,8 @@ def combine_by_coords(
                 join=join,
                 combine_attrs=combine_attrs,
             )
-            concatenated_grouped_by_data_vars.append(concatenated)
+            for vars, datasets_with_same_vars in grouped_by_vars
+        )
 
     return merge(
         concatenated_grouped_by_data_vars,
