@@ -1,40 +1,37 @@
 from __future__ import annotations
 
-from typing import (
-    TYPE_CHECKING,
-    AbstractSet,
-    Any,
-    Hashable,
-    Iterable,
-    Mapping,
-    NamedTuple,
-    Optional,
-    Sequence,
-    Tuple,
-    Union,
-)
+from collections import defaultdict
+from collections.abc import Hashable, Iterable, Mapping, Sequence, Set
+from typing import TYPE_CHECKING, Any, NamedTuple, Optional, Union
 
 import pandas as pd
 
-from . import dtypes
-from .alignment import deep_align
-from .duck_array_ops import lazy_array_equiv
-from .indexes import Index, PandasIndex
-from .utils import Frozen, compat_dict_union, dict_equiv, equivalent
-from .variable import Variable, as_variable, assert_unique_multiindex_level_names
+from xarray.core import dtypes
+from xarray.core.alignment import deep_align
+from xarray.core.duck_array_ops import lazy_array_equiv
+from xarray.core.indexes import (
+    Index,
+    Indexes,
+    create_default_index_implicit,
+    filter_indexes_from_coords,
+    indexes_equal,
+)
+from xarray.core.utils import Frozen, compat_dict_union, dict_equiv, equivalent
+from xarray.core.variable import Variable, as_variable, calculate_dimensions
 
 if TYPE_CHECKING:
-    from .coordinates import Coordinates
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.coordinates import Coordinates
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
+    from xarray.core.types import CombineAttrsOptions, CompatOptions, JoinOptions
 
     DimsLike = Union[Hashable, Sequence[Hashable]]
     ArrayLike = Any
     VariableLike = Union[
         ArrayLike,
-        Tuple[DimsLike, ArrayLike],
-        Tuple[DimsLike, ArrayLike, Mapping],
-        Tuple[DimsLike, ArrayLike, Mapping, Mapping],
+        tuple[DimsLike, ArrayLike],
+        tuple[DimsLike, ArrayLike, Mapping],
+        tuple[DimsLike, ArrayLike, Mapping, Mapping],
     ]
     XarrayValue = Union[DataArray, Variable, VariableLike]
     DatasetLike = Union[Dataset, Mapping[Any, XarrayValue]]
@@ -87,8 +84,8 @@ class MergeError(ValueError):
 def unique_variable(
     name: Hashable,
     variables: list[Variable],
-    compat: str = "broadcast_equals",
-    equals: bool = None,
+    compat: CompatOptions = "broadcast_equals",
+    equals: bool | None = None,
 ) -> Variable:
     """Return the unique variable from a list of variables or raise MergeError.
 
@@ -162,14 +159,47 @@ def _assert_compat_valid(compat):
         raise ValueError(f"compat={compat!r} invalid: must be {set(_VALID_COMPAT)}")
 
 
-MergeElement = Tuple[Variable, Optional[Index]]
+MergeElement = tuple[Variable, Optional[Index]]
+
+
+def _assert_prioritized_valid(
+    grouped: dict[Hashable, list[MergeElement]],
+    prioritized: Mapping[Any, MergeElement],
+) -> None:
+    """Make sure that elements given in prioritized will not corrupt any
+    index given in grouped.
+    """
+    prioritized_names = set(prioritized)
+    grouped_by_index: dict[int, list[Hashable]] = defaultdict(list)
+    indexes: dict[int, Index] = {}
+
+    for name, elements_list in grouped.items():
+        for _, index in elements_list:
+            if index is not None:
+                grouped_by_index[id(index)].append(name)
+                indexes[id(index)] = index
+
+    # An index may be corrupted when the set of its corresponding coordinate name(s)
+    # partially overlaps the set of names given in prioritized
+    for index_id, index_coord_names in grouped_by_index.items():
+        index_names = set(index_coord_names)
+        common_names = index_names & prioritized_names
+        if common_names and len(common_names) != len(index_names):
+            common_names_str = ", ".join(f"{k!r}" for k in common_names)
+            index_names_str = ", ".join(f"{k!r}" for k in index_coord_names)
+            raise ValueError(
+                f"cannot set or update variable(s) {common_names_str}, which would corrupt "
+                f"the following index built from coordinates {index_names_str}:\n"
+                f"{indexes[index_id]!r}"
+            )
 
 
 def merge_collected(
-    grouped: dict[Hashable, list[MergeElement]],
-    prioritized: Mapping[Any, MergeElement] = None,
-    compat: str = "minimal",
-    combine_attrs="override",
+    grouped: dict[Any, list[MergeElement]],
+    prioritized: Mapping[Any, MergeElement] | None = None,
+    compat: CompatOptions = "minimal",
+    combine_attrs: CombineAttrsOptions = "override",
+    equals: dict[Any, bool] | None = None,
 ) -> tuple[dict[Hashable, Variable], dict[Hashable, Index]]:
     """Merge dicts of variables, while resolving conflicts appropriately.
 
@@ -179,6 +209,24 @@ def merge_collected(
     prioritized : mapping
     compat : str
         Type of equality check to use when checking for conflicts.
+    combine_attrs : {"drop", "identical", "no_conflicts", "drop_conflicts", \
+                    "override"} or callable, default: "override"
+        A callable or a string indicating how to combine attrs of the objects being
+        merged:
+
+        - "drop": empty attrs on returned Dataset.
+        - "identical": all attrs must be the same on every object.
+        - "no_conflicts": attrs from all objects are combined, any that have
+          the same name must also have the same value.
+        - "drop_conflicts": attrs from all objects are combined, any that have
+          the same name but different values are dropped.
+        - "override": skip comparing and copy attrs from the first dataset to
+          the result.
+
+        If a callable, it must expect a sequence of ``attrs`` dicts and a context object
+        as its only parameters.
+    equals : mapping, optional
+        corresponding to result of compat test
 
     Returns
     -------
@@ -188,11 +236,15 @@ def merge_collected(
     """
     if prioritized is None:
         prioritized = {}
+    if equals is None:
+        equals = {}
 
     _assert_compat_valid(compat)
+    _assert_prioritized_valid(grouped, prioritized)
 
     merged_vars: dict[Hashable, Variable] = {}
     merged_indexes: dict[Hashable, Index] = {}
+    index_cmp_cache: dict[tuple[int, int], bool | None] = {}
 
     for name, elements_list in grouped.items():
         if name in prioritized:
@@ -206,17 +258,19 @@ def merge_collected(
                 for variable, index in elements_list
                 if index is not None
             ]
-
             if indexed_elements:
                 # TODO(shoyer): consider adjusting this logic. Are we really
                 # OK throwing away variable without an index in favor of
                 # indexed variables, without even checking if values match?
                 variable, index = indexed_elements[0]
-                for _, other_index in indexed_elements[1:]:
-                    if not index.equals(other_index):
+                for other_var, other_index in indexed_elements[1:]:
+                    if not indexes_equal(
+                        index, other_index, variable, other_var, index_cmp_cache
+                    ):
                         raise MergeError(
-                            f"conflicting values for index {name!r} on objects to be "
-                            f"combined:\nfirst value: {index!r}\nsecond value: {other_index!r}"
+                            f"conflicting values/indexes on objects to be combined fo coordinate {name!r}\n"
+                            f"first index: {index!r}\nsecond index: {other_index!r}\n"
+                            f"first variable: {variable!r}\nsecond variable: {other_var!r}\n"
                         )
                 if compat == "identical":
                     for other_variable, _ in indexed_elements[1:]:
@@ -234,7 +288,9 @@ def merge_collected(
             else:
                 variables = [variable for variable, _ in elements_list]
                 try:
-                    merged_vars[name] = unique_variable(name, variables, compat)
+                    merged_vars[name] = unique_variable(
+                        name, variables, compat, equals.get(name, None)
+                    )
                 except MergeError:
                     if compat != "minimal":
                         # we need more than "minimal" compatibility (for which
@@ -250,7 +306,8 @@ def merge_collected(
 
 
 def collect_variables_and_indexes(
-    list_of_mappings: list[DatasetLike],
+    list_of_mappings: Iterable[DatasetLike],
+    indexes: Mapping[Any, Any] | None = None,
 ) -> dict[Hashable, list[MergeElement]]:
     """Collect variables and indexes from list of mappings of xarray objects.
 
@@ -260,15 +317,21 @@ def collect_variables_and_indexes(
     - a tuple `(dims, data[, attrs[, encoding]])` that can be converted in
       an xarray.Variable
     - or an xarray.DataArray
-    """
-    from .dataarray import DataArray
-    from .dataset import Dataset
 
-    grouped: dict[Hashable, list[tuple[Variable, Index | None]]] = {}
+    If a mapping of indexes is given, those indexes are assigned to all variables
+    with a matching key/name.
+
+    """
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
+
+    if indexes is None:
+        indexes = {}
+
+    grouped: dict[Hashable, list[MergeElement]] = defaultdict(list)
 
     def append(name, variable, index):
-        values = grouped.setdefault(name, [])
-        values.append((variable, index))
+        grouped[name].append((variable, index))
 
     def append_all(variables, indexes):
         for name, variable in variables.items():
@@ -276,27 +339,26 @@ def collect_variables_and_indexes(
 
     for mapping in list_of_mappings:
         if isinstance(mapping, Dataset):
-            append_all(mapping.variables, mapping.xindexes)
+            append_all(mapping.variables, mapping._indexes)
             continue
 
         for name, variable in mapping.items():
             if isinstance(variable, DataArray):
-                coords = variable._coords.copy()  # use private API for speed
-                indexes = dict(variable.xindexes)
+                coords_ = variable._coords.copy()  # use private API for speed
+                indexes_ = dict(variable._indexes)
                 # explicitly overwritten variables should take precedence
-                coords.pop(name, None)
-                indexes.pop(name, None)
-                append_all(coords, indexes)
+                coords_.pop(name, None)
+                indexes_.pop(name, None)
+                append_all(coords_, indexes_)
 
             variable = as_variable(variable, name=name)
-
-            if variable.dims == (name,):
-                idx_variable = variable.to_index_variable()
-                index = variable._to_xindex()
-                append(name, idx_variable, index)
+            if name in indexes:
+                append(name, variable, indexes[name])
+            elif variable.dims == (name,):
+                idx, idx_vars = create_default_index_implicit(variable)
+                append_all(idx_vars, {k: idx for k in idx_vars})
             else:
-                index = None
-                append(name, variable, index)
+                append(name, variable, None)
 
     return grouped
 
@@ -305,22 +367,22 @@ def collect_from_coordinates(
     list_of_coords: list[Coordinates],
 ) -> dict[Hashable, list[MergeElement]]:
     """Collect variables and indexes to be merged from Coordinate objects."""
-    grouped: dict[Hashable, list[tuple[Variable, Index | None]]] = {}
+    grouped: dict[Hashable, list[MergeElement]] = defaultdict(list)
 
     for coords in list_of_coords:
         variables = coords.variables
         indexes = coords.xindexes
         for name, variable in variables.items():
-            value = grouped.setdefault(name, [])
-            value.append((variable, indexes.get(name)))
+            grouped[name].append((variable, indexes.get(name)))
+
     return grouped
 
 
 def merge_coordinates_without_align(
     objects: list[Coordinates],
-    prioritized: Mapping[Any, MergeElement] = None,
-    exclude_dims: AbstractSet = frozenset(),
-    combine_attrs: str = "override",
+    prioritized: Mapping[Any, MergeElement] | None = None,
+    exclude_dims: Set = frozenset(),
+    combine_attrs: CombineAttrsOptions = "override",
 ) -> tuple[dict[Hashable, Variable], dict[Hashable, Index]]:
     """Merge variables/indexes from coordinates without automatic alignments.
 
@@ -342,7 +404,14 @@ def merge_coordinates_without_align(
     else:
         filtered = collected
 
-    return merge_collected(filtered, prioritized, combine_attrs=combine_attrs)
+    # TODO: indexes should probably be filtered in collected elements
+    # before merging them
+    merged_coords, merged_indexes = merge_collected(
+        filtered, prioritized, combine_attrs=combine_attrs
+    )
+    merged_indexes = filter_indexes_from_coords(merged_indexes, set(merged_coords))
+
+    return merged_coords, merged_indexes
 
 
 def determine_coords(
@@ -362,8 +431,8 @@ def determine_coords(
         All variable found in the input should appear in either the set of
         coordinate or non-coordinate names.
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
     coord_names: set[Hashable] = set()
     noncoord_names: set[Hashable] = set()
@@ -397,8 +466,8 @@ def coerce_pandas_values(objects: Iterable[CoercibleMapping]) -> list[DatasetLik
     List of Dataset or dictionary objects. Any inputs or values in the inputs
     that were pandas objects have been converted into native xarray objects.
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
     out = []
     for obj in objects:
@@ -407,7 +476,7 @@ def coerce_pandas_values(objects: Iterable[CoercibleMapping]) -> list[DatasetLik
         else:
             variables = {}
             if isinstance(obj, PANDAS_TYPES):
-                obj = dict(obj.iteritems())
+                obj = dict(obj.items())
             for k, v in obj.items():
                 if isinstance(v, PANDAS_TYPES):
                     v = DataArray(v)
@@ -417,7 +486,9 @@ def coerce_pandas_values(objects: Iterable[CoercibleMapping]) -> list[DatasetLik
 
 
 def _get_priority_vars_and_indexes(
-    objects: list[DatasetLike], priority_arg: int | None, compat: str = "equals"
+    objects: list[DatasetLike],
+    priority_arg: int | None,
+    compat: CompatOptions = "equals",
 ) -> dict[Hashable, MergeElement]:
     """Extract the priority variable from a list of mappings.
 
@@ -431,8 +502,19 @@ def _get_priority_vars_and_indexes(
         Dictionaries in which to find the priority variables.
     priority_arg : int or None
         Integer object whose variable should take priority.
-    compat : {"identical", "equals", "broadcast_equals", "no_conflicts"}, optional
-        Compatibility checks to use when merging variables.
+    compat : {"identical", "equals", "broadcast_equals", "no_conflicts", "override"}, optional
+        String indicating how to compare non-concatenated variables of the same name for
+        potential conflicts. This is passed down to merge.
+
+        - "broadcast_equals": all values must be equal when variables are
+          broadcast against each other to ensure common dimensions.
+        - "equals": all values and dimensions must be the same.
+        - "identical": all values, dimensions and attributes must be the
+          same.
+        - "no_conflicts": only values which are not null in both datasets
+          must be equal. The returned dataset then contains the combination
+          of all non-null values.
+        - "override": skip comparing and pick variable from first dataset
 
     Returns
     -------
@@ -451,8 +533,8 @@ def _get_priority_vars_and_indexes(
 
 def merge_coords(
     objects: Iterable[CoercibleMapping],
-    compat: str = "minimal",
-    join: str = "outer",
+    compat: CompatOptions = "minimal",
+    join: JoinOptions = "outer",
     priority_arg: int | None = None,
     indexes: Mapping[Any, Index] | None = None,
     fill_value: object = dtypes.NA,
@@ -468,32 +550,73 @@ def merge_coords(
     aligned = deep_align(
         coerced, join=join, copy=False, indexes=indexes, fill_value=fill_value
     )
-    collected = collect_variables_and_indexes(aligned)
+    collected = collect_variables_and_indexes(aligned, indexes=indexes)
     prioritized = _get_priority_vars_and_indexes(aligned, priority_arg, compat=compat)
     variables, out_indexes = merge_collected(collected, prioritized, compat=compat)
-    assert_unique_multiindex_level_names(variables)
     return variables, out_indexes
 
 
-def merge_data_and_coords(data, coords, compat="broadcast_equals", join="outer"):
+def merge_data_and_coords(
+    data_vars: Mapping[Any, Any],
+    coords: Mapping[Any, Any],
+    compat: CompatOptions = "broadcast_equals",
+    join: JoinOptions = "outer",
+) -> _MergeResult:
     """Used in Dataset.__init__."""
-    objects = [data, coords]
+    indexes, coords = _create_indexes_from_coords(coords, data_vars)
+    objects = [data_vars, coords]
     explicit_coords = coords.keys()
-    indexes = dict(_extract_indexes_from_coords(coords))
     return merge_core(
-        objects, compat, join, explicit_coords=explicit_coords, indexes=indexes
+        objects,
+        compat,
+        join,
+        explicit_coords=explicit_coords,
+        indexes=Indexes(indexes, coords),
     )
 
 
-def _extract_indexes_from_coords(coords):
-    """Yields the name & index of valid indexes from a mapping of coords"""
-    for name, variable in coords.items():
-        variable = as_variable(variable, name=name)
+def _create_indexes_from_coords(
+    coords: Mapping[Any, Any], data_vars: Mapping[Any, Any] | None = None
+) -> tuple[dict, dict]:
+    """Maybe create default indexes from a mapping of coordinates.
+
+    Return those indexes and updated coordinates.
+    """
+    all_variables = dict(coords)
+    if data_vars is not None:
+        all_variables.update(data_vars)
+
+    indexes = {}
+    updated_coords = {}
+
+    # this is needed for backward compatibility: when a pandas multi-index
+    # is given as data variable, it is promoted as index / level coordinates
+    # TODO: depreciate this implicit behavior
+    index_vars = {
+        k: v
+        for k, v in all_variables.items()
+        if k in coords or isinstance(v, pd.MultiIndex)
+    }
+
+    for name, obj in index_vars.items():
+        variable = as_variable(obj, name=name)
+
         if variable.dims == (name,):
-            yield name, variable._to_xindex()
+            idx, idx_vars = create_default_index_implicit(variable, all_variables)
+            indexes.update({k: idx for k in idx_vars})
+            updated_coords.update(idx_vars)
+            all_variables.update(idx_vars)
+        else:
+            updated_coords[name] = obj
+
+    return indexes, updated_coords
 
 
-def assert_valid_explicit_coords(variables, dims, explicit_coords):
+def assert_valid_explicit_coords(
+    variables: Mapping[Any, Any],
+    dims: Mapping[Any, int],
+    explicit_coords: Iterable[Hashable],
+) -> None:
     """Validate explicit coordinate names/dims.
 
     Raise a MergeError if an explicit coord shares a name with a dimension
@@ -566,17 +689,17 @@ class _MergeResult(NamedTuple):
     variables: dict[Hashable, Variable]
     coord_names: set[Hashable]
     dims: dict[Hashable, int]
-    indexes: dict[Hashable, pd.Index]
+    indexes: dict[Hashable, Index]
     attrs: dict[Hashable, Any]
 
 
 def merge_core(
     objects: Iterable[CoercibleMapping],
-    compat: str = "broadcast_equals",
-    join: str = "outer",
-    combine_attrs: str | None = "override",
+    compat: CompatOptions = "broadcast_equals",
+    join: JoinOptions = "outer",
+    combine_attrs: CombineAttrsOptions = "override",
     priority_arg: int | None = None,
-    explicit_coords: Sequence | None = None,
+    explicit_coords: Iterable[Hashable] | None = None,
     indexes: Mapping[Any, Any] | None = None,
     fill_value: object = dtypes.NA,
 ) -> _MergeResult:
@@ -620,8 +743,8 @@ def merge_core(
     ------
     MergeError if the merge cannot be done successfully.
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset, calculate_dimensions
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
     _assert_compat_valid(compat)
 
@@ -629,13 +752,11 @@ def merge_core(
     aligned = deep_align(
         coerced, join=join, copy=False, indexes=indexes, fill_value=fill_value
     )
-    collected = collect_variables_and_indexes(aligned)
-
+    collected = collect_variables_and_indexes(aligned, indexes=indexes)
     prioritized = _get_priority_vars_and_indexes(aligned, priority_arg, compat=compat)
     variables, out_indexes = merge_collected(
         collected, prioritized, compat=compat, combine_attrs=combine_attrs
     )
-    assert_unique_multiindex_level_names(variables)
 
     dims = calculate_dimensions(variables)
 
@@ -663,10 +784,10 @@ def merge_core(
 
 def merge(
     objects: Iterable[DataArray | CoercibleMapping],
-    compat: str = "no_conflicts",
-    join: str = "outer",
+    compat: CompatOptions = "no_conflicts",
+    join: JoinOptions = "outer",
     fill_value: object = dtypes.NA,
-    combine_attrs: str = "override",
+    combine_attrs: CombineAttrsOptions = "override",
 ) -> Dataset:
     """Merge any number of xarray objects into a single Dataset as variables.
 
@@ -675,20 +796,23 @@ def merge(
     objects : iterable of Dataset or iterable of DataArray or iterable of dict-like
         Merge together all variables from these objects. If any of them are
         DataArray objects, they must have a name.
-    compat : {"identical", "equals", "broadcast_equals", "no_conflicts", "override"}, optional
+    compat : {"identical", "equals", "broadcast_equals", "no_conflicts", \
+              "override", "minimal"}, default: "no_conflicts"
         String indicating how to compare variables of the same name for
         potential conflicts:
 
-        - "broadcast_equals": all values must be equal when variables are
-          broadcast against each other to ensure common dimensions.
-        - "equals": all values and dimensions must be the same.
         - "identical": all values, dimensions and attributes must be the
           same.
+        - "equals": all values and dimensions must be the same.
+        - "broadcast_equals": all values must be equal when variables are
+          broadcast against each other to ensure common dimensions.
         - "no_conflicts": only values which are not null in both datasets
           must be equal. The returned dataset then contains the combination
           of all non-null values.
         - "override": skip comparing and pick variable from first dataset
-    join : {"outer", "inner", "left", "right", "exact"}, optional
+        - "minimal": drop conflicting coordinates
+
+    join : {"outer", "inner", "left", "right", "exact", "override"}, default: "outer"
         String indicating how to combine differing indexes in objects.
 
         - "outer": use the union of object indexes
@@ -700,12 +824,13 @@ def merge(
         - "override": if indexes are of same size, rewrite indexes to be
           those of the first object with that dimension. Indexes for the same
           dimension must have the same size in all objects.
+
     fill_value : scalar or dict-like, optional
         Value to use for newly missing values. If a dict-like, maps
         variable names to fill values. Use a data array's name to
         refer to its values.
     combine_attrs : {"drop", "identical", "no_conflicts", "drop_conflicts", \
-                    "override"} or callable, default: "override"
+                     "override"} or callable, default: "override"
         A callable or a string indicating how to combine attrs of the objects being
         merged:
 
@@ -870,7 +995,7 @@ def merge(
     >>> xr.merge([x, y, z], join="exact")
     Traceback (most recent call last):
     ...
-    ValueError: indexes along dimension 'lat' are not equal
+    ValueError: cannot align objects with join='exact' where ...
 
     Raises
     ------
@@ -883,8 +1008,8 @@ def merge(
     combine_nested
     combine_by_coords
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
     dict_like_objects = []
     for obj in objects:
@@ -911,17 +1036,17 @@ def dataset_merge_method(
     dataset: Dataset,
     other: CoercibleMapping,
     overwrite_vars: Hashable | Iterable[Hashable],
-    compat: str,
-    join: str,
+    compat: CompatOptions,
+    join: JoinOptions,
     fill_value: Any,
-    combine_attrs: str,
+    combine_attrs: CombineAttrsOptions,
 ) -> _MergeResult:
     """Guts of the Dataset.merge method."""
     # we are locked into supporting overwrite_vars for the Dataset.merge
     # method due for backwards compatibility
     # TODO: consider deprecating it?
 
-    if isinstance(overwrite_vars, Iterable) and not isinstance(overwrite_vars, str):
+    if not isinstance(overwrite_vars, str) and isinstance(overwrite_vars, Iterable):
         overwrite_vars = set(overwrite_vars)
     else:
         overwrite_vars = {overwrite_vars}
@@ -960,8 +1085,8 @@ def dataset_update_method(dataset: Dataset, other: CoercibleMapping) -> _MergeRe
     `xarray.Dataset`, e.g., if it's a dict with DataArray values (GH2068,
     GH2180).
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
     if not isinstance(other, Dataset):
         other = dict(other)
@@ -976,18 +1101,9 @@ def dataset_update_method(dataset: Dataset, other: CoercibleMapping) -> _MergeRe
                 if coord_names:
                     other[key] = value.drop_vars(coord_names)
 
-    # use ds.coords and not ds.indexes, else str coords are cast to object
-    # TODO: benbovy - flexible indexes: make it work with any xarray index
-    indexes = {}
-    for key, index in dataset.xindexes.items():
-        if isinstance(index, PandasIndex):
-            indexes[key] = dataset.coords[key]
-        else:
-            indexes[key] = index
-
     return merge_core(
         [dataset, other],
         priority_arg=1,
-        indexes=indexes,  # type: ignore
+        indexes=dataset.xindexes,
         combine_attrs="override",
     )
